use actix_web::{web, HttpResponse, Responder};
use sqlx::PgPool;

use crate::{
    endpoints::IdPath,
    models::{Event, Role, User},
    utils::{self, ApplicationError},
};

#[actix_web::put("/events/{id}/cancel")]
pub async fn put_cancel(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
    path: web::Path<IdPath>,
) -> Result<impl Responder, ApplicationError> {
    handle_set_event_cancelation_to(user, pool, path, true).await
}

#[actix_web::put("/events/{id}/uncancel")]
pub async fn put_uncancel(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
    path: web::Path<IdPath>,
) -> Result<impl Responder, ApplicationError> {
    handle_set_event_cancelation_to(user, pool, path, false).await
}

async fn handle_set_event_cancelation_to(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
    path: web::Path<IdPath>,
    cancelation_state: bool,
) -> Result<impl Responder, ApplicationError> {
    if user.role != Role::Admin && user.role != Role::AreaManager {
        return Err(ApplicationError::Unauthorized);
    }

    let Some(event) = Event::read_by_id_including_location(pool.get_ref(), path.id).await? else {
        return Ok(HttpResponse::NotFound().finish());
    };

    if event.canceled != cancelation_state {
        if user.role != Role::Admin && user.area_id != event.location.as_ref().unwrap().area_id {
            return Ok(HttpResponse::BadRequest().body("Can't use location outside of your area"));
        }

        Event::update_cancelation(pool.get_ref(), event.id, cancelation_state).await?;
    }

    let url = utils::get_return_url_for_date(&event.start.date());
    Ok(HttpResponse::Ok()
        .insert_header(("HX-LOCATION", url))
        .finish())
}
